@@ -13,6 +13,10 @@ mclustBootstrapLRT <- function(data, modelName = NULL,
   if(is.null(modelName))
     stop("A 'modelName' must be provided. Please see help(mclustModelNames) which describes the available models.")
   modelName <- modelName[1]
+  checkModelName(modelName)
+  if(grepl("X", modelName))
+    stop("Specified 'modelName' is only valid for one-component mixture.")
+  
   if(is.null(maxG))
   { 
     G <- seq.int(1, 9)
@@ -21,6 +25,7 @@ mclustBootstrapLRT <- function(data, modelName = NULL,
     maxG <- as.numeric(maxG)
     G <- seq.int(1, maxG+1) 
   }
+  
   BIC <- mclustBIC(data, G = G, modelNames = modelName, 
                    warn = FALSE, verbose = FALSE, ...)
   
@@ -139,8 +144,8 @@ MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "j
                             max.nonfit = 10*nboot, verbose = interactive(), ...)
 {
   
-  if(!any(class(object) %in% c("Mclust", "densityMclust")))
-    stop("object must be of class 'Mclust' or 'densityMclust'")
+	stopifnot("object must be of class 'Mclust' or 'densityMclust'" =
+	          inherits(object, c("Mclust", "densityMclust")))
   
   if(any(type %in% c("nonpara", "wlb")))
     { type <- gsub("nonpara", "bs", type)
