@@ -9,21 +9,20 @@
 
 # GMMDR dimension reduction -----------------------------------------------
 
-MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(.Machine$double.eps))
+MclustDR <- function(object, lambda = 1,
+                     normalized = TRUE, Sigma, 
+                     tol = sqrt(.Machine$double.eps))
 {
-  #  Dimension reduction for model-based clustering and classification
-  #
-  # object = a object of class '"Mclust' 
-  # data = the data used to produce object.
-  # normalized = normalize direction coefs to have unit norm
+#  Dimension reduction for model-based clustering and classification
+
+	stopifnot("first argument must be an object of class 'Mclust' or 'MclustDA'" = 
+            inherits(object, c("Mclust", "MclustDA")))
   
   call <- match.call()
-  if(!any(class(object) %in% c("Mclust", "MclustDA")))
-    stop("object must be of class 'Mclust' or 'MclustDA'")
-  
   x <- data.matrix(object$data)
-  p <- ncol(x)
   n <- nrow(x)
+  p <- ncol(x)
+  lambda <- pmax(0, min(lambda, 1))
   #-----------------------------------------------------------------  
   # overall parameters
   mu <- colMeans(x)
@@ -45,9 +44,12 @@ MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(
     mu.G <- matrix(par$mean,p,G) 
     # within-group covars
     if(p == 1) 
-    { Sigma.G <- array(par$variance$sigmasq, c(p,p,G)) }
-    else     
-    { Sigma.G <- par$variance$sigma }
+    { 
+      Sigma.G <- array(par$variance$sigmasq, c(p,p,G)) 
+    } else     
+    { 
+      Sigma.G <- par$variance$sigma 
+    }
   }
   else if(inherits(object, "MclustDA"))
   { 
@@ -57,8 +59,10 @@ MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(
     class <- factor(class, levels = names(object$models))
     y <- rep(NA, length(class))
     for(i in 1:nlevels(class))
-    { y[class == levels(class)[i]] <- paste(levels(class)[i], 
-                                            object$models[[i]]$classification, sep =":") }
+    { 
+      y[class == levels(class)[i]] <- paste(levels(class)[i], 
+                                            object$models[[i]]$classification, sep =":") 
+    }
     y <- as.numeric(factor(y))
     cl2mc <- rep(seq(length(object$models)), 
                  sapply(object$models, function(m) m$G))
@@ -85,35 +89,34 @@ MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(
     }
   }
   #-----------------------------------------------------------------
-  SVD <- svd(Sigma)
-  pos <- (SVD$d > max(tol*SVD$d[1], 0)) # in case of not full rank covar matrix
-  if(all(pos)) 
-  { inv.Sigma <- SVD$v %*% (1/SVD$d * t(SVD$u))
-    inv.sqrt.Sigma <- SVD$v %*% (1/sqrt(SVD$d) * t(SVD$u)) }
-  else
-  { inv.Sigma <- SVD$v[,pos,drop=FALSE] %*% 
-      (1/SVD$d[pos] * t(SVD$u[,pos,drop=FALSE]))
-    inv.sqrt.Sigma <- SVD$v[,pos,drop=FALSE] %*% 
-      (1/sqrt(SVD$d[pos]) * t(SVD$u[,pos,drop=FALSE])) }
+  SVD <- svd(Sigma, nu = 0, nv = min(n,p))
+  pos <- which(SVD$d > max(tol*SVD$d[1], 0)) 
+  SVD$d <- SVD$d[pos]
+  SVD$v <- SVD$v[,pos,drop=FALSE]
+  inv.Sigma <- SVD$v %*% (1/SVD$d * t(SVD$v))
+  inv.sqrt.Sigma <- SVD$v %*% (1/sqrt(SVD$d) * t(SVD$v)) 
   #-----------------------------------------------------------------
   # pooled within-group covariance
   S <- matrix(0, p, p)
-  for(j in 1:G) 
+  for(j in seq_len(G))
     S <- S + f[j]*Sigma.G[,,j]
   #-----------------------------------------------------------------
   # kernel matrix
   M.I <- crossprod(t(sweep(mu.G, 1, FUN="-", STATS=mu))*sqrt(f))
   M.II <- matrix(0, p, p)
-  for(j in 1:G)
-    M.II <- M.II + f[j]*crossprod(inv.sqrt.Sigma%*%(Sigma.G[,,j]-S))
-  # convex combiation of M_I and M_II
+  if(lambda < 1) 
+  {
+    for(j in seq_len(G))
+      M.II <- M.II + f[j]*crossprod(inv.sqrt.Sigma%*%(Sigma.G[,,j]-S))
+  }
+  # convex combination of M_I and M_II
   M <- 2*lambda*crossprod(inv.sqrt.Sigma %*% M.I) + 2*(1-lambda)*M.II
   # regularize the M_II
   # M <- M.I + lambda*M.II
   # M <- crossprod(inv.sqrt.Sigma %*% M.I) + 
   #      (1-lambda)*M.II + lambda/p * diag(p)  
   #
-  SVD <- eigen.decomp(M, Sigma)
+  SVD <- eigen.decomp(M, inv.sqrt.Sigma, invsqrt = TRUE)
   l <- SVD$l; l <- (l+abs(l))/2
   numdir <- min(p, sum(l > sqrt(.Machine$double.eps)))
   basis <- as.matrix(SVD$v)[,seq(numdir),drop=FALSE]
@@ -129,7 +132,8 @@ MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(
   #
   out = list(call = call, type = type,
              x = x, Sigma = Sigma, 
-             class = class, mixcomp = y, class2mixcomp = cl2mc,
+             classification = class, mixcomp = y, 
+             class2mixcomp = cl2mc,
              G = G, modelName = modelName,
              mu = mu.G, sigma = Sigma.G, pro = f,
              M = M, M.I = M.I, M.II = M.II, 
@@ -141,32 +145,32 @@ MclustDR <- function(object, normalized = TRUE, Sigma, lambda = 0.5, tol = sqrt(
   return(out)
 }
 
-print.MclustDR <- function(x, ...) 
+print.MclustDR <- function(x, digits = getOption("digits"), ...)
 {
-  cat(paste("\'", class(x), "\' object for ", x$type, 
-            " mixture model:\n\n", sep = ""))
-  tab <- rbind(x$basis[,seq(x$numdir),drop=FALSE],
-               "-----------" = rep(NA, x$numdir), 
-               Eigenvalues = x$evalues[seq(x$numdir)])
-  print(tab, na.print = "", ...)
-  invisible()
+  txt <- paste0("\'", class(x)[1], "\' model object: ")
+  catwrap(txt)
+  cat("\n")
+  catwrap("\nAvailable components:\n")
+  print(names(x))
+  # str(x, max.level = 1, give.attr = FALSE, strict.width = "wrap")
+  invisible(x)
 }
-
+  
 summary.MclustDR <- function(object, numdir, std = FALSE, ...)
 {
   if(missing(numdir)) numdir <- object$numdir
   dim <- seq(numdir)
   
   if(object$type == "Mclust")
-  { n <- as.vector(table(object$class))
+  { n <- as.vector(table(object$classification))
     G <- object$G }
   else
-  { n <- as.vector(table(object$class))
+  { n <- as.vector(table(object$classification))
     G <- as.vector(table(object$class2mixcomp)) }
   
   obj <- list(type = object$type, 
               modelName = object$modelName, 
-              classes = levels(object$class),
+              classes = levels(object$classification),
               n = n, G = G, 
               basis = object$basis[,seq(dim),drop=FALSE],
               std = std, std.basis = object$std.basis[,seq(dim),drop=FALSE],
@@ -182,18 +186,20 @@ return(obj)
 print.summary.MclustDR <- function(x, digits = max(5, getOption("digits") - 3), ...)
 {
   title <- paste("Dimension reduction for model-based clustering and classification")
-  cat(rep("-", nchar(title)),"\n",sep="")
-  cat(title, "\n")
-  cat(rep("-", nchar(title)),"\n",sep="")
-  
+  txt <- paste(rep("-", min(nchar(title), getOption("width"))), collapse = "")
+  catwrap(txt)
+  catwrap(title)
+  catwrap(txt)
+
   if(x$type == "Mclust")
   { 
     tab <- data.frame(n = x$n)
     rownames(tab) <- x$classes
     tab <- as.matrix(tab)
     names(dimnames(tab)) <- c("Clusters", "")
-    cat(paste("\nMixture model type: ", x$type, 
-              " (", x$modelName, ", ", x$G, ")\n", sep = ""))
+    cat("\n")
+    catwrap(paste0("Mixture model type: ", x$type, 
+                   " (", x$modelName, ", ", x$G, ")"))
     print(tab, quote = FALSE, right = TRUE)
   }
   else if(x$type == "MclustDA" | x$type == "EDDA")
@@ -202,18 +208,21 @@ print.summary.MclustDR <- function(x, digits = max(5, getOption("digits") - 3),
     rownames(tab) <- x$classes
     tab <- as.matrix(tab)
     names(dimnames(tab)) <- c("Classes", "")
-    cat(paste("\nMixture model type:", x$type, "\n"))
+    cat("\n")
+    catwrap(paste("Mixture model type:", x$type))
     print(tab, quote = FALSE, right = TRUE)
   }
   else stop("invalid model type")
   
+  cat("\n")
   if(x$std) 
-  { cat("\nStandardized basis vectors",  "using predictors", "scaled to have", 
-        "std.dev. equal to one:\n", fill = TRUE)
+  { 
+    catwrap("Standardized basis vectors using predictors scaled to have std.dev. equal to one:")
     print(x$std.basis, digits = digits)
   }
   else 
-  { cat("\nEstimated basis vectors:\n")
+  { 
+    catwrap("Estimated basis vectors:")
     print(x$basis, digits = digits)
   }
   cat("\n")
@@ -267,7 +276,7 @@ predict.MclustDR <- function(object, dim = 1:object$numdir, newdata, eval.points
   
   n <- nrow(dir)
   G <- object$G # num. components
-  nclass <- nlevels(object$class) # num. classes
+  nclass <- nlevels(object$classification) # num. classes
   par <- projpar.MclustDR(object, dim)
   Mu <- par$mean
   Sigma <- par$variance
@@ -275,14 +284,14 @@ predict.MclustDR <- function(object, dim = 1:object$numdir, newdata, eval.points
   # old version
   # cden <- array(NA, c(n, G))
   # for(j in 1:G)
-  #    { cden[,j] <- mvdnorm(dir, Mu[j,], Sigma[,,j], log = FALSE) }
+  #    { cden[,j] <- dmvnorm(dir, Mu[j,], Sigma[,,j], log = FALSE) }
   # z <- sweep(cden, 2, FUN = "*", STATS = object$pro)  
   # den <- apply(z, 1, sum)
   # z <- sweep(z, 1, FUN = "/", STATS = den)
   # new version: more efficient and accurate
   z <- array(NA, c(n, G))
   for(j in 1:G)
-     { z[,j] <- mvdnorm(dir, Mu[j,], Sigma[,,j], log = TRUE) }
+     { z[,j] <- dmvnorm(dir, Mu[j,], Sigma[,,j], log = TRUE) }
   z <- sweep(z, 2, FUN = "+", STATS = log(object$pro))
   logden <- apply(z, 1, logsumexp)
   z <- sweep(z, 1, FUN = "-", STATS = logden)
@@ -295,7 +304,7 @@ predict.MclustDR <- function(object, dim = 1:object$numdir, newdata, eval.points
 
   class <- factor(apply(z,1,which.max), 
                   levels = 1:nclass, 
-                  labels = levels(object$class))
+                  labels = levels(object$classification))
   
   out <- list(dir = dir,
               density = exp(logden),
@@ -329,7 +338,16 @@ predict2D.MclustDR <- function(object, dim = 1:2, ngrid = 100, xlim, ylim)
   return(out)
 }
 
-plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour", "classification", "boundaries", "density", "evalues"), symbols, colors, col.contour = gray(0.7), col.sep = grey(0.4), ngrid = 100, nlevels = 5, asp = NULL, ...)
+plot.MclustDR <- function(x, dimens, 
+                          what = c("scatterplot", "pairs", "contour", 
+                                   "classification", "boundaries", 
+                                   "density", "evalues"), 
+                          symbols, colors, 
+                          col.contour = gray(0.7), 
+                          col.sep = grey(0.4), 
+                          ngrid = 200, 
+                          nlevels = 5, 
+                          asp = NULL, ...)
 { 
   object <- x
   x <- object$x
@@ -337,12 +355,14 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
   n <- nrow(x)
   G <- object$G
   y <- object$mixcomp
-  class <- as.numeric(object$class)
+  class <- as.numeric(object$classification)
   nclass <- length(table(class))
   dir <- object$dir
   numdir <- object$numdir
   dimens <- if(missing(dimens)) seq(numdir)
             else intersect(as.numeric(dimens), seq(numdir))
+  if(length(dimens) == 0)
+    stop("invalid 'dimens' value(s) provided")
   
   if(missing(symbols)) 
     { if(G <= length(mclust.options("classPlotSymbols"))) 
@@ -362,14 +382,6 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
     { warning("more colors needed to show mixture components")
       colors <- rep("black", nclass) }
   
-  niceRange <- function (x, f = 0.04) 
-  { 
-    r <- range(x)
-    d <- diff(r)
-    out <- c(r[1] - d*f, r[2] + d*f)
-    return(out)
-  }
-
   ####################################################################
   what <- match.arg(what, several.ok = TRUE)
   oldpar <- par(no.readonly = TRUE)
@@ -377,7 +389,9 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
   if(any(i <- (what == "pairs")) & (length(dimens) == 2))
     { what[i] <- "scatterplot" }
   if(length(dimens) == 1) 
-    { what[!(what == "density" | what == "evalues")] <- "density" }
+    { what[!(what == "classification" | 
+             what == "density" | 
+             what == "evalues")] <- "classification" }
   what <- unique(what)
   
   plot.MclustDR.scatterplot <- function(...)
@@ -389,47 +403,41 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
   }
   
   plot.MclustDR.pairs <- function(...)
-  { dir <- dir[,dimens,drop=FALSE]
+  { 
+    dir <- dir[,dimens,drop=FALSE]
     pairs(dir, col = colors[class], pch = symbols[class], 
           gap = 0.2, asp = asp, ...)
   }
   
   plot.MclustDR.density <- function(...)
-  { dimens <- dimens[1]
+  { 
+    dimens <- dimens[1]
     dir <- object$dir[,dimens,drop=FALSE]
     par <- projpar.MclustDR(object, dimens)
     Mu <- par$mean
     Sigma <- par$variance
-    q <- seq(min(dir), max(dir), length=2*ngrid)
-    dens <- matrix(as.double(NA), length(q), G)
+    xgrid <- extendrange(dir, f = 0.1)
+    xgrid <- seq(min(xgrid), max(xgrid), length=2*ngrid)
+    dens <- matrix(as.double(NA), length(xgrid), G)
     for(j in 1:G)
-      dens[,j] <- dnorm(q, Mu[j,], sqrt(Sigma[,,j]))
+      dens[,j] <- dnorm(xgrid, Mu[j,], sqrt(Sigma[,,j]))
     #
     if(object$type == "MclustDA")
-    { d <- t(apply(dens, 1, function(x, p = object$pro) p*x))
-      dens <- matrix(as.double(NA), length(q), nclass) 
+    { 
+      d <- t(apply(dens, 1, function(x, p = object$pro) p*x))
+      dens <- matrix(as.double(NA), length(xgrid), nclass) 
       tab <- table(y, class)
       for(i in 1:ncol(tab))
-      { j <- which(tab[,i] > 0)
+      { 
+        j <- which(tab[,i] > 0)
         dens[,i] <- apply(d[,j,drop=FALSE],1,sum) 
       }
     }
-    #
-    oldpar <- par(mar = c(0,5.1,1,1), mfrow = par("mfrow"), no.readonly = TRUE)
-    on.exit(par(oldpar))
-    layout(matrix(1:2,2,1), heights = c(2,1))
     plot(0, 0, type = "n", xlab = colnames(dir), ylab = "Density", 
-         xlim = range(q, dir), ylim = range(0, dens*1.1), xaxt = "n")
+         xlim = range(xgrid), ylim = range(0, dens*1.1))
     for(j in 1:ncol(dens))
-      lines(q, dens[,j], col = colors[j])
-    dir.class <- split(dir, class) 
-    par(mar = c(4.1,5.1,0,1))
-    boxplot(dir.class, col = adjustcolor(colors[1:nclass], alpha.f = 0.3),
-            border = colors[1:nclass], horizontal = TRUE, 
-            pars = list(boxwex = 0.6, staplewex = 0.8, medlwd = 2,
-                        whisklty = 3, outlty = 1, outpch = NA),
-            ylim = range(q,dir), yaxt = "n", xlab = colnames(dir))
-    axis(2, at = 1:nclass, labels = levels(object$class), tick = FALSE, cex = 0.8, las = 2)
+      lines(xgrid, dens[,j], col = colors[j])
+    rug(dir, lwd = 1)
   }
   
   plot.MclustDR.contour <- function(...)
@@ -457,22 +465,29 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
   }
   
   plot.MclustDR.classification.Mclust <- function(...)
-  { dimens <- dimens[1:2]
+  { 
+    if(object$numdir == 1)
+    {
+      dir <- object$dir[,1]
+      boxplot(dir ~ class, horizontal = TRUE, 
+              col = adjustcolor(mclust.options("classPlotColors"), alpha.f = 0.3)[1:nclass],
+              border = mclust.options("classPlotColors")[1:nclass],
+              ylab = "Classification", xlab = "Dir1",
+              xlim = c(0,nclass+0.5))
+      points(dir, rep(0,n), pch = "|")
+      return()
+    }
+    # numdir >= 2
+    dimens <- dimens[1:2]
     dir <- object$dir[,dimens,drop=FALSE]
     pred <- predict2D.MclustDR(object, dimens, ngrid,
-                               xlim = niceRange(dir[,1]), 
-                               ylim = niceRange(dir[,2]))
+                               xlim = extendrange(dir[,1], f = 0.05), 
+                               ylim = extendrange(dir[,2], f = 0.05))
     pred$classification <- apply(pred$z, 1:2, which.max)
-    #
     image(pred$x, pred$y, pred$classification, 
           col = adjustcolor(colors[1:G], alpha.f = 0.1),
           xlab = colnames(dir)[1], ylab = colnames(dir)[2],
           xaxs = "i", yaxs = "i", asp = asp)
-    # for(j in 1:G)         
-    #    { z <- ifelse(pred$classification == j, 1, -1)
-    #      contour(pred$x, pred$y, z, col = col.sep,
-    #              add = TRUE, levels = 0, drawlabels = FALSE) 
-    # }
     points(dir, col = colors[class], pch = symbols[class], ...)
   }
   
@@ -481,29 +496,26 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
     dimens <- dimens[1:2]
     dir <- object$dir[,dimens,drop=FALSE]
     pred <- predict2D.MclustDR(object, dimens, ngrid,
-                               xlim = niceRange(dir[,1]), 
-                               ylim = niceRange(dir[,2]))
+                               xlim = extendrange(dir[,1], f = 0.05), 
+                               ylim = extendrange(dir[,2], f = 0.05))
     pred$classification <- apply(pred$z, 1:2, which.max)
-    #
     image(pred$x, pred$y, pred$classification, 
           col = adjustcolor(colors[1:nclass], alpha.f = 0.1),
           xlab = colnames(dir)[1], ylab = colnames(dir)[2],
           xaxs = "i", yaxs = "i", asp = asp)
-    # for(j in 1:nclass)
-    #    { z <- ifelse(pred$classification == j, 1, -1)
-    #      contour(pred$x, pred$y, z, col = col.sep,
-    #              add = TRUE, levels = 0, drawlabels = FALSE) 
-    # }
     points(dir, col = colors[class], pch = symbols[class], ...)
   }
   
   plot.MclustDR.boundaries.Mclust <- function(...)
-  { dimens <- dimens[1:2]
+  { 
+    dimens <- dimens[1:2]
     dir <- object$dir[,dimens,drop=FALSE]
     pred <- predict2D.MclustDR(object, dimens, ngrid,
-                               xlim = niceRange(dir[,1]), 
-                               ylim = niceRange(dir[,2]))
-    image(pred$x, pred$y, pred$uncertainty, 
+                               xlim = extendrange(dir[,1], f = 0.05), 
+                               ylim = extendrange(dir[,2], f = 0.05))
+    pred$uncertainty[c(1,ngrid),] <- NA
+    pred$uncertainty[,c(1,ngrid)] <- NA
+    image(pred$x, pred$y, pred$uncertainty,
           col = rev(gray.colors(10, start = 0, end = 1)),
           breaks = seq(0, 1-1/nclass, length = 11),
           xlab = colnames(dir)[1], ylab = colnames(dir)[2],
@@ -516,20 +528,23 @@ plot.MclustDR <- function(x, dimens, what = c("scatterplot", "pairs", "contour",
     dimens <- dimens[1:2]
     dir <- object$dir[,dimens,drop=FALSE]
     pred <- predict2D.MclustDR(object, dimens, ngrid,
-                               xlim = niceRange(dir[,1]), 
-                               ylim = niceRange(dir[,2]))
-    levels <- seq(0, 1-1/nclass, length = 11)
-    col <- rev(gray.colors(10, start = 0, end = 1))
+                               xlim = extendrange(dir[,1], f = 0.05), 
+                               ylim = extendrange(dir[,2], f = 0.05))
+    pred$uncertainty[c(1,ngrid),] <- NA
+    pred$uncertainty[,c(1,ngrid)] <- NA
     image(pred$x, pred$y, pred$uncertainty,
-          col = col, breaks = levels,
+          col = rev(gray.colors(10, start = 0, end = 1)),
+          breaks = seq(0, 1-1/nclass, length = 11),
           xlab = colnames(dir)[1], ylab = colnames(dir)[2],
           xaxs = "i", yaxs = "i", asp = asp)
     points(dir, col = colors[class], pch = symbols[class], ...)
   }
   
   plot.MclustDR.evalues <- function(...)
-  { plotEvalues.MclustDR(object, numdir = max(dimens), plot = TRUE) }
-  
+  { 
+    plotEvalues.MclustDR(object, numdir = max(dimens), plot = TRUE) 
+  }
+
   if(interactive() & length(what) > 1)
     { title <- "Dimension reduction for model-based clustering and classification plots:"
       # present menu waiting user choice
@@ -617,8 +632,8 @@ plotEvalues.MclustDR <- function(x, numdir, plot = FALSE, legend = TRUE, ylim, .
     if(legend)
     { legend("topright", lty = c(1,2,2), pch = c(16,22,2), 
              legend = c("Eigenvalues", 
-                        "Means contrib.", 
-                        "Vars contrib."),
+                        "from means", 
+                        "from vars"),
              bg = ifelse(par("bg")=="transparent", "white", par("bg")),
              inset = 0.01, pt.cex = 1.5) }
   }
@@ -631,23 +646,24 @@ plotEvalues.MclustDR <- function(x, numdir, plot = FALSE, legend = TRUE, ylim, .
 
 # Auxiliary functions -----------------------------------------------------
 
-mvdnorm <- function(x, mu, sigma, log = FALSE, tol = sqrt(.Machine$double.eps))
-{
-  if(is.vector(x)) 
-  { x <- matrix(x, ncol = length(x)) }
-  else
-  { x <- as.matrix(x) }
-  SVD <- svd(sigma)
-  pos <- (SVD$d > max(tol*SVD$d[1], 0)) # in case of not full rank covar matrix
-  inv.sigma <- SVD$v[,pos,drop=FALSE] %*% (1/SVD$d[pos] *
-                                             t(SVD$u[,pos,drop=FALSE]))
-  z <- mahalanobis(x, center = mu, cov = inv.sigma, inverted = TRUE)
-  # logdet <- sum(log(eigen(sigma, symmetric = TRUE, only.values = TRUE)$values))
-  logdet <- sum(log(SVD$d[pos]))
-  logdens <- -(ncol(x) * log(2 * pi) + logdet + z)/2
-  if(log) return(logdens)
-  else    return(exp(logdens))
-}
+# TODO: remove
+# mvdnorm <- function(x, mu, sigma, log = FALSE, tol = sqrt(.Machine$double.eps))
+# {
+#   if(is.vector(x)) 
+#   { x <- matrix(x, ncol = length(x)) }
+#   else
+#   { x <- as.matrix(x) }
+#   SVD <- svd(sigma)
+#   pos <- (SVD$d > max(tol*SVD$d[1], 0)) # in case of not full rank covar matrix
+#   inv.sigma <- SVD$v[,pos,drop=FALSE] %*% (1/SVD$d[pos] *
+#                                              t(SVD$u[,pos,drop=FALSE]))
+#   z <- mahalanobis(x, center = mu, cov = inv.sigma, inverted = TRUE)
+#   # logdet <- sum(log(eigen(sigma, symmetric = TRUE, only.values = TRUE)$values))
+#   logdet <- sum(log(SVD$d[pos]))
+#   logdens <- -(ncol(x) * log(2 * pi) + logdet + z)/2
+#   if(log) return(logdens)
+#   else    return(exp(logdens))
+# }
 
 ellipse <- function(c, M, r, npoints = 100)
 {
@@ -667,38 +683,46 @@ ellipse <- function(c, M, r, npoints = 100)
   return(coord)
 }
 
-eigen.decomp <- function(A, B)
+eigen.decomp <- function(A, B, invsqrt = FALSE)
 {
-  #
-  # Generalized eigenvalue decomposition of A with respect to B
-  #
-  # A generalized eigenvalue problem AV = BLV is said to be symmetric positive 
-  # definite if A is symmetric and B is positive definite. V is the matrix of
-  # generalized eigenvectors, and L is the diagonal matrix of generalized 
-  # eigenvalues (Stewart, 2001, pag. 229-230).
-  # 
-  # Properties:
-  # V'AV = L
-  # V'BV = I
-  #
-  # The algorithm implemented is described in Stewart (2001, pag. 234) and used
-  # by Li (2000).
-  #
-  # References: 
-  # Li, K.C., 2000. High dimensional data analysis via the SIR-PHD approach,
-  # Stewart, G.W., 2001. Matrix Algorithms: vol II Eigensystems, SIAM.
+#
+# Generalized eigenvalue decomposition of A with respect to B.
+#
+# A generalized eigenvalue problem AV = BLV is said to be symmetric positive 
+# definite if A is symmetric and B is positive definite. V is the matrix of
+# generalized eigenvectors, and L is the diagonal matrix of generalized 
+# eigenvalues (Stewart, 2001, pag. 229-230).
+# 
+# Properties:
+# V'AV = L
+# V'BV = I
+#
+# The algorithm implemented is described in Stewart (2001, pag. 234) and used
+# by Li (2000).
+#
+# References: 
+# Li, K.C., 2000. High dimensional data analysis via the SIR-PHD approach,
+# Stewart, G.W., 2001. Matrix Algorithms: vol II Eigensystems, SIAM.
   
-  svd <- svd(B, nu=0)
-  p <- length(svd$d)
-  # Computes inverse square root matrix such that: 
-  # t(inv.sqrt.B) %*% inv.sqrt.B = inv.sqrt.B %*% t(inv.sqrt.B) = solve(B)
-  inv.sqrt.B <- svd$v %*% diag(1/sqrt(svd$d),p,p) %*% t(svd$v)
+  if(!invsqrt)
+  {
+    SVD <- svd(B, nu=0)
+    # in case of not full rank covar matrix
+    tol <- .Machine$double.eps
+    pos <- which(SVD$d > max(tol*SVD$d[1], 0)) 
+    SVD$d <- SVD$d[pos]
+    SVD$v <- SVD$v[,pos,drop=FALSE]
+    # Computes inverse square root matrix such that: 
+    # t(inv.sqrt.B) %*% inv.sqrt.B = inv.sqrt.B %*% t(inv.sqrt.B) = solve(B)
+    inv.sqrt.B <- SVD$v %*% (1/sqrt(SVD$d) * t(SVD$v))
+  } else
+  { inv.sqrt.B <-  B }
   # Compute  B^(-1/2)' A B^(-1/2) = UDU'
   # evectors = B^(-1/2) U 
   # evalues  = D
   A <- t(inv.sqrt.B) %*% A %*% inv.sqrt.B
-  svd <- svd(A, nu=0)
-  list(l = svd$d, v = inv.sqrt.B  %*% svd$v)
+  SVD <- svd(A, nu=0)
+  list(l = SVD$d, v = inv.sqrt.B  %*% SVD$v)
 }
 
 
@@ -730,9 +754,9 @@ MclustDRsubsel <- function(object, G = 1:9,
 #              of the search is printed; if 2 a detailed trace info is
 #              is shown.
 
-  if(class(object) != "MclustDR") 
-    stop("Not a 'MclustDR' object")
-  
+  stopifnot("first argument must be an object of class 'MclustDR'" = 
+            inherits(object, "MclustDR"))
+						  
   hcUse <- mclust.options("hcUse")
   mclust.options("hcUse" = "VARS")
   on.exit(mclust.options("hcUse" = hcUse))  
@@ -800,8 +824,8 @@ MclustDRsubsel1cycle <- function(object,
 {
 # Single cycle of subset selection for GMMDR directions based on bayes factors.
 
-  if(class(object) != "MclustDR") 
-    stop("Not a 'MclustDR' object")
+  stopifnot("first argument must be an object of class 'MclustDR'" = 
+            inherits(object, "MclustDR"))
 
   d <- object$numdir
   dir <- object$dir[,seq(d),drop=FALSE]
@@ -889,7 +913,7 @@ MclustDRsubsel_classif <- function(object,
                                  verbose = if(verbose > 1) TRUE else FALSE)
     if(verbose > 0) { cat("\n"); print(out$tab) }
     mod <- do.call("MclustDA", list(data = dir[,out$subset,drop=FALSE], 
-                                    class = object$class,
+                                    class = object$classification,
                                     G = G, 
                                     modelNames = if(length(out$subset) > 1) modelNames
                                                  else if(any(grep("V", modelNames))) 
@@ -920,8 +944,8 @@ MclustDRCsubsel1cycle <- function(object,
 {
 # Single cycle of subset selection for GMMDRC directions based on bayes factors.
 
-  if(class(object) != "MclustDR") 
-    stop("Not a 'MclustDR' object")
+  stopifnot("first argument must be an object of class 'MclustDR'" = 
+            inherits(object, "MclustDR"))
 
   d <- object$numdir
   dir <- object$dir[,seq(d),drop=FALSE]
@@ -946,7 +970,7 @@ MclustDRCsubsel1cycle <- function(object,
             { # Select simplest model with smallest num. of components 
               # within bic.cutoff
               mod <- MclustDA(dir[,sort(c(inc, j)),drop=FALSE], 
-                              class = object$class,
+                              class = object$classification,
                               G = G, 
                               modelNames = if(length(inc)>0) modelNames else model1D,
                               modelType = object$type,
@@ -1025,8 +1049,9 @@ MclustDRrecoverdir <- function(object, data, normalized = TRUE, std = FALSE)
 # directions. This is useful if the directions are obtained from other 
 # directions
 
-  if(!any(class(object) == "MclustDR"))
-    stop("object must be of class mclustsir")
+  stopifnot("first argument must be an object of class 'MclustDR'" = 
+            inherits(object, "MclustDR"))
+						
   if(missing(data)) x <- object$x
   else              x <- as.matrix(data)
   x <- scale(x, center=TRUE, scale=FALSE)
